# frozen_string_literal: true

module Hermes
  class HousingRequestShare
    include Hermes::Concerns::CommonFieldsConcern

    field :code, type: String
    field :expire_at, type: DateTime

    belongs_to :housing_request, index: true, class_name: 'Hermes::HousingRequest'
    belongs_to :lessor, index: true, class_name: 'Hermes::Lessor'

    validates :housing_request, presence: true
    validates :lessor, presence: true

    after_create :after_create_action

    common_indexes *%i[
      code
      expire_at
    ]

    def self.find_or_create_by(attributes={})
      instance = super(attributes)
      instance.refresh_code
      instance
    end

    def generate_code(length = 12)
      (0...length).map { ('a'..'z').to_a[rand(26)] }.join
    end

    def generate_expiration(days_count = 14)
      DateTime.current.beginning_of_day + days_count.days
    end

    def public_url
      Hermes::Engine.routes.url_helpers.api_dmo_housing_request_share_path(id: id, code: code)
    end

    def after_create_action
      set(code: generate_code, expire_at: generate_expiration, updated_at: DateTime.current)
    end

    def refresh_code
      set(expire_at: generate_expiration, updated_at: DateTime.current)
    end
  end
end
