require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::ContactGroupsController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers
  let!(:responsible) {create(:user, roles: ['Administrator'])}
  let!(:superadministrateur) {create(:user, auth_roles: ['superadministrateur'])}
  let!(:user) {create(:user)}

  #TESTS BASIQUES D'API

  describe 'POST /contact_groups' do
    describe 'Create contact_group with minimal valid datas' do
      before :each do
        authenticated_header(request, responsible)
        post 'create', {
            data: {
                attributes: {
                }
            }
        }
      end
      it 'returns status code 201' do
        expect(response).to have_http_status(201)
      end
    end

    #contact_ids:'Array'&& main_contact_id:'String',
    describe 'post request with contact_ids && main_contact_id' do
      let!(:premier) {create(:doc, type: 'contact')}
      let!(:deuxieme) {create(:doc, type: 'contact')}
      before :each do
        authenticated_header(request, responsible)
        post 'create', {
            data: {
                attributes: {
                    contact_ids: [premier.id.to_s, deuxieme.id.to_s],
                    main_contact_id: premier.id.to_s
                }
            }
        }
      end
      it 'returns status code 201 and the value appears in json' do
        expect(response).to have_http_status(201)
        expect(json['data']['attributes']['contact_ids']).to eq([premier.id.to_s, deuxieme.id.to_s])
        expect(json['data']['attributes']['main_contact_id']).to eq(premier.id.to_s)
      end
    end

    # entity_id:'String',
    describe 'post request with entity_id' do
      let!(:entity) {create(:entity)}
      before :each do
        authenticated_header(request, responsible)
        post 'create', {
            data: {
                attributes: {
                    entity_id: entity.id.to_s
                }
            }
        }
      end
      it 'returns status code 201 and the value appears in json' do
        expect(response).to have_http_status(201)
        expect(json['data']['attributes']['entity_id']).to eq(entity.id.to_s)
      end
    end
    #lessor_id:'String'
    describe 'post request with lessor_id' do
      let!(:lessor) {create(:lessor)}
      before :each do
        authenticated_header(request, responsible)
        post 'create', {
            data: {
                attributes: {
                    lessor_id: lessor.id.to_s
                }
            }
        }
      end
      it 'returns status code 201 and the value appears in json' do
        expect(response).to have_http_status(201)
        expect(json['data']['attributes']['lessor_id']).to eq(lessor.id.to_s)
      end
    end
  end

  describe 'GET /contact_groups' do
    let!(:contact_group1) {create(:contact_group)}
    let!(:contact_group3) {create(:contact_group)}
    let!(:contact_group2) {create(:contact_group)}

    [
      {user: 'responsible', total_objects: 3},
      {user: 'superadministrateur', total_objects: 3},
      {user: 'user', total_objects: 0},
    ].each do |a| 
      describe "as #{a[:user]}" do

        before :each do
          authenticated_header(request, eval(a[:user]))
          get 'index'
        end
        it 'returns status code 200' do
          expect(response).to have_http_status(200)
        end
        it 'returns a valid json' do
          expect(json).not_to be_empty
          expect(json['data'].length).to eq(a[:total_objects])
          expect(json['meta']['pagination']).to be_present
          expect(json['meta']['pagination']['total_objects']).to eq(a[:total_objects])
        end
      end
    end

  end

  describe 'GET /contact_groups/:id' do
    describe 'verif it work' do
      let!(:contact_group1) {create(:contact_group)}
      before :each do
        authenticated_header(request, responsible)
        get 'show', {id: contact_group1.id}
      end
      it 'returns status code 200' do
        expect(response).to have_http_status(200)
      end
      it 'returns a valid json' do
        expect(json).not_to be_empty
      end
    end
    describe 'verif it work not work with a non authorized user' do
      let!(:contact_group1) {create(:contact_group)}

      before :each do
        authenticated_header(request, user)
        get 'show', {id: contact_group1.id}
      end
      it 'returns status code 404' do
        expect(response).to have_http_status(404)
      end
    end
  end

  describe 'PATCH /contact_groups/:id' do

  end

  describe 'DELETE /contact_groups/:id' do
    let!(:contact_group) {create(:contact_group, user_id: responsible.id.to_s)}
    before :each do
      authenticated_header(request, responsible)

      delete 'destroy', {
          id: contact_group.id.to_s
      }
    end
    it 'returns status code 204' do
      expect(response).to have_http_status(204)
    end
  end

end

require_relative '../../swagger_helper'

# describe Hermes::Api::Dmo::ContactGroupsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
#   TAGS_API_DMO_CONTACT_GROUPS = 'API_DMO_CONTACT_GROUPS'

#   
#   extend HermesSwaggerJsonParams
#   include SwaggerJsonAttributes
#   let!(:responsible) {create(:user, first_name: 'Martin', last_name: 'Durand', email: 'martin.durand@gmail.com', roles: ['Administrator'])}
#   let!(:user) {create(:user, first_name: 'Jackie', last_name: 'Mike', email: 'jackie.mike@gmail.com')}

#   context 'as admin' do
#     let!(:admin) {create(:administrator)}

#     path '/api/dmo/contact_groups' do
#       post 'Create contact_group' do
#         tags tags_API_DMO_CONTACT_GROUPS
#         produces 'application/json'
#         consumes 'application/json'
#         security [Bearer: {}]
#         json_params = {}
#         MUTATE_PARAMS.each do |key, val|
#           json_params[key] = {type: nice_type(val)}
#         end
#         parameter name: :params, in: :body, schema: jsonify_params(json_params)
#         #jsonapi_params( **json_params )

#         response '201', 'contact_group created' do
#           let(:Authorization) {"Bearer #{token_for(responsible)}"}
#           let(:params) do
#             jsonapi_attributes(
#                {})
#           end
#           include_context 'with integration test'
#           include_context 'valid jsonapi'
#           include_context 'not containings errors'

#           after do |_example|
#             json = JSON.parse(response.body)

#             expect(json['data']).to be_a Hash
#             expect(json['data']['id']).to_not be nil
#             expect(json['data']['type']).to eq 'contact_groups'
#           end
#         end
#       end
#     end

#     path '/api/dmo/contact_groups' do
#       get 'Retrieves all contact_groups' do
#         tags tags_API_DMO_CONTACT_GROUPS
#         produces 'application/json'

#         parameter name: 'ids[]', in: :query, type: :array, items: { type: :string }, collectionFormat: :multi, required: false
#         ACCEPTABLE_PARAMS.each do |acceptable_param|
#           type = ''
#           if MUTATE_PARAMS.include?(acceptable_param)
#             type = nice_type(MUTATE_PARAMS[acceptable_param])
#           end
#           parameter name: acceptable_param.to_s, in: :query, type: type, required: false
#         end
#         # VALID_OPERATORS.each do |valid_operator|
#         #   parameter( name: valid_operator.to_s, in: :query,type: :object, required: false)
#         # end

#         security [Bearer: {}]
#         paginator

#         response '200', 'contact_groups found' do
#           let(:Authorization) {"Bearer #{token_for(admin)}"}
#           let!(:contact_groups) {create_list(:contact_group, 5)}

#           include_context 'with integration test'
#           include_context 'valid jsonapi collection' do
#             let(:expected_collection) {contact_groups}
#           end
#         end
#       end
#     end

#     path '/api/dmo/contact_groups/{id}' do
#       get 'Retrieves a single contact_group by id' do
#         tags tags_API_DMO_CONTACT_GROUPS
#         produces 'application/json'
#         parameter name: :id, in: :path, type: :string
#         security [Bearer: {}]

#         response '200', 'contact_group found' do
#           let(:Authorization) {"Bearer #{token_for(admin)}"}
#           let(:contact_group) {create(:contact_group)}
#           let(:id) {contact_group.id}

#           include_context 'with integration test'
#           include_context 'valid jsonapi ressource' do
#             let(:expected_ressource) {contact_group}
#           end
#         end
#       end
#     end

#     path '/api/dmo/contact_groups/{id}' do
#       put 'edit contact_group' do
#         tags tags_API_DMO_CONTACT_GROUPS
#         produces 'application/json'
#         consumes 'application/json'
#         parameter name: :id, in: :path, type: :string
#         security [Bearer: {}]
#         json_params = {}
#         MUTATE_PARAMS.each do |key, val|
#           json_params[key] = {type: nice_type(val)}
#         end
#         parameter name: :params, in: :body, schema: jsonify_params(json_params)
#         #jsonapi_params( **json_params )

#         response '200', 'contact_group updated' do
#           let(:Authorization) {"Bearer #{token_for(admin)}"}
#           let(:contact_group) {create(:contact_group)}
#           let(:id) {contact_group.id}
#           let(:params) do
#             jsonapi_attributes(
#                { lessor_id: 'String'}
#             )
#           end
#           include_context 'with integration test'
#           include_context 'valid jsonapi'
#           include_context 'not containings errors'

#           after do |_example|
          
#             json = JSON.parse(response.body)

#             expect(json['data']).to be_a Hash
#             expect(json['data']['id']).to_not be nil
#             expect(json['data']['type']).to eq 'contact_groups'
#           end
#         end
#       end
#     end

#     path '/api/dmo/contact_groups/{id}' do
#       delete 'Delete a single contact_group by id' do
#         tags tags_API_DMO_CONTACT_GROUPS
#         produces 'application/json'
#         parameter name: :id, in: :path, type: :string
#         security [Bearer: {}]

#         response '204', 'contact_group deleted' do
#           let(:Authorization) {"Bearer #{token_for(admin)}"}
#           let(:id) {create(:contact_group, responsible: admin.id).id}
#           run_test!
#         end
#       end
#     end
#   end
# end