require 'rails_helper'
require [Hermes::Engine.root, %w[lib hermes tools specs]].flatten.join('/')

describe Hermes::Api::Dmo::UpdateSubscriptionsController, :type => :api do
  routes {Hermes::Engine.routes}
  include Devise::Test::ControllerHelpers

  let!(:superadministrateur) { create(:user, auth_roles: ['superadministrateur']) }
  let!(:user) { create(:user) }

  before do 
    ['key.num1', 'key.num2', 'key.num3'].each do |k|
      superadministrateur.subscription.subscribe_to_email(k)
      user.subscription.subscribe_to_email(k)
    end 
  end 

  describe 'GET /update_subscriptions' do
    describe 'as superadministrateur' do 
      it ' works with with filtering' do
        authenticated_header(request, superadministrateur)
        get 'index', {name: { '$in': [ 'key.num1'], object_id: { '$in': [ superadministrateur.id.to_s, user.id.to_s]}}}
        expect(response).to have_http_status(200)
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(2)
      end
    end

    describe 'as user' do 
      it 'returns status code 200' do
        authenticated_header(request, user)
        get 'index'
        expect(response).to have_http_status(200)
        expect(json).not_to be_empty
        expect(json['data'].length).to eq(4)
      end
    end
  end
end

require_relative '../../swagger_helper'

describe Hermes::Api::Dmo::UpdateSubscriptionsController, type: :request, swagger_doc: 'api/swagger/hermes/swagger_hermes.json' do
  tags = 'API_DMO_UPDATE_SUBSCRIPTIONS'
  
  extend HermesSwaggerJsonParams
  include SwaggerJsonAttributes
  let!(:admin) { create(:user, roles: ['Administrator']) }

  path '/api/dmo/update_subscriptions' do
    get 'Retrieves all update_subscriptions' do
      tags tags
      produces 'application/json'

      Hermes::Api::Dmo::UpdateSubscriptionsController.acceptable_params.sort.each do |params_name, options|
        parameter name: params_name.to_s, in: options[:in] || :query, type: options[:type], required: options[:required] || false, description: options[:description] || ''
      end

      security [Bearer: {}]
      paginator

      response '401', 'unauthorized' do
        let(:Authorization) { "Bearer" }

        run_test!
      end 

      response '200', 'update_subscriptions found' do
        let(:Authorization) { "Bearer #{token_for(admin)}" }
        let!(:update_subscriptions) { create_list(:update_subscription, 1) }

        run_test!
      end
    end
  end
end
