require 'rails_helper'

RSpec.describe Hermes::UpdateSeniorityPointJob do
  subject { Hermes::UpdateSeniorityPointJob }
  let!(:hr) {create(:complete_request)}

  it 'should add job at hr creation' do
    expect( enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.length).to eq(1)
  end

  it 'should launch an other one' do
    HERMES_CONFIG['update_seniority_point_job'] = true
    Timecop.freeze(hr.calculate_next_seniority_updated_date)
    first_job = enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.first
    first_job[:job].perform_now(*first_job[:args])
    expect(hr.reload.score_details['seniority_points']).to eq(1)
    expect(enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.length).to eq(2)
    
    second_job = enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.last
    expect( first_job[:at]).to_not eq(second_job[:at])
    
    Timecop.freeze( hr.reload.calculate_next_seniority_updated_date)
    second_job[:job].perform_now(*second_job[:args])
    third_job = enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.last
    
    expect( hr.reload.score_details['seniority_points']).to eq(2)
    expect(enqueued_jobs.select {|job| job[:job] == UpdateSeniorityPointJob}.length).to eq(3)
    expect( third_job[:at]).to_not eq(second_job[:at])
    Timecop.return

    HERMES_CONFIG['update_seniority_point_job'] = nil
  end 

end
