# frozen_string_literal: true

require 'rails_helper'
require 'vcr_config'
require [Hermes::Engine.root, %w[lib hermes rex_client tasks]].flatten.join('/').to_s
require [Hermes::Engine.root, %w[lib hermes rex_client]].flatten.join('/').to_s

RSpec.describe Hermes::RexClient::UpdateStructureTask, type: :helper do
  let!(:entity) { create(:entity, title: 'MON AGENCE', id_cristal: '119057') }
  let!(:entity_parent) { create(:entity, title: 'MON AGENCE PARENT', id_cristal: '299785') }

  describe 'Existing entity' do
    it 'update entity and create parent' do
      VCR.use_cassette("rex_update_existing_entity", :match_requests_on => [:method, :uri, :path]) do
        client = Hermes::RexClient::Client.new
        rex_entity = client.search_entities({'fields.id_cristal' => entity.id_cristal}).first
        Hermes::RexClient::UpdateStructureTask.new.update_entity(rex_entity, 'Hermes::Entity', client, false)
      end
      expect(entity.reload.title).to eq "SUEZ RV SUD OUEST"
      expect(Hermes::Entity.all().count).to eq(2)
      expect(Hermes::Entity.find_by(id: entity.reload.parent_id).title).to eq("MON AGENCE PARENT")
    end
  end

  describe 'Not existing entity' do
    it 'create entity and parent' do
      VCR.use_cassette("rex_create_new_entity", :match_requests_on => [:method, :uri, :path]) do
        client = Hermes::RexClient::Client.new
        rex_entity = client.search_entities({'fields.id_cristal' => '264301'}).first
        Hermes::RexClient::UpdateStructureTask.new.update_entity(rex_entity, 'Hermes::Entity', client)
      end
      created_entity = Hermes::Entity.find_by(id_cristal: '264301')
      expect(Hermes::Entity.all().count).to eq(4)
      expect(created_entity.title).to eq "DIJON CEREALES MEUNERIE"
      expect(Hermes::Entity.find_by(id: created_entity.reload.parent_id).title).to eq("AXEREAL SCA")
    end
  end
end
